# webcam-uploader.py
# by Markus Gritsch (gritsch@iue.tuwien.ac.at)

"""Uploades a picture obtained by video-capturing to the net periodically.

The destination of the picture and the interval between successive uploads is
specified in the webcam-uploader.ini file.  Further details are given there.
"""

import ConfigParser, StringIO, ftplib, time, os, sys, string
import VideoCapture

def upload():
    fp = None
    ftp = None
    try:
        # get an image from the webcam
        im = cam.getImage(timestamp=3, boldfont=1)

        # save it to a string buffer
        fp = StringIO.StringIO()
        im.save(fp, format, quality=quality)
        fp.flush()
        fp.seek(0)

        # establish an FTP connection and upload it
        if verbose:
            print 'uploading ...',
        ftp = ftplib.FTP(host, user, passwd)
        ftp.storbinary('STOR ' + filename, fp, 8192)
        if firsttime:
            ftp.sendcmd('SITE CHMOD 644 ' + filename)
        if verbose:
            print 'done at', time.asctime(time.localtime(time.time()))
    except IOError:
        type, value = sys.exc_info()[:2]
        print 'webcam-uploader.py: ' + str(type) + ':', value
    if ftp:
        ftp.close()
    if fp:
        fp.close()

parser = ConfigParser.ConfigParser()
parser.read('webcam-uploader.ini')
host = parser.get('uploader', 'host')
user = parser.get('uploader', 'user')
passwd = parser.get('uploader', 'passwd')
interval = int(parser.get('uploader', 'interval'))
devnum = int(parser.get('uploader', 'devnum'))
filename = parser.get('uploader', 'filename')
quality = int(parser.get('uploader', 'quality'))
verbose = int(parser.get('uploader', 'verbose'))
if not passwd:
    import getpass
    passwd = getpass.getpass()

root, ext = os.path.splitext(filename)
head, tail = os.path.split(root)

temp = """<HTML>
  <HEAD>
    <TITLE>-{IMGNAME}-</TITLE>
    <META HTTP-EQUIV="Refresh" CONTENT="-{INTERVAL}-;URL=-{HTMLURL}-">
    <META HTTP-EQUIV="Pragma" CONTENT="no-cache">
    <SCRIPT LANGUAGE="JavaScript">
      function doreload()
      {
        location.reload()
        window.setTimeout("doreload()",-{JSINTERVAL}-);
      }
      window.setTimeout("doreload()",-{JSINTERVAL}-);
    </SCRIPT>
  </HEAD>
  <BODY BGCOLOR=#ffffff>
    <IMG SRC="-{IMGSRC}-" NAME="-{IMGNAME}-"><BR>
    powered by <A HREF="http://stud4.tuwien.ac.at/~e9326522/VideoCapture/">VideoCapture</A>
  </BODY>
</HTML>
"""

temp = string.replace(temp, '-{INTERVAL}-', str(interval))
temp = string.replace(temp, '-{HTMLURL}-', tail + '.html')
temp = string.replace(temp, '-{JSINTERVAL}-', str(interval * 1000 + 1000))
temp = string.replace(temp, '-{IMGSRC}-', tail + ext)
temp = string.replace(temp, '-{IMGNAME}-', tail)

print 'uploading HTML-file ' + root + '.html ...',
fp = StringIO.StringIO(temp)
ftp = ftplib.FTP(host, user, passwd)
ftp.storlines('STOR ' + root + '.html', fp)
ftp.sendcmd('SITE CHMOD 644 ' + root + '.html')
ftp.close()
fp.close()
print 'done'

# determine the desired image format
ext = ext.lower()
if ext == '.jpg' or ext == '.jpeg':
    format = 'JPEG'
elif ext == '.gif':
    format = 'GIF'
elif ext == '.bmp':
    format = 'BMP'
else:
    raise ValueError, 'unsupportet image format'

print '''
Press Ctrl+C to exit gracefully.
Do not terminate this program by closing the window.
'''

cam = VideoCapture.Device(devnum=devnum)
firsttime = 1

while 1:
    try:
        recenttime = time.time()
        upload()
        firsttime = 0
        while time.time() - recenttime < interval:
            time.sleep(1)
    except KeyboardInterrupt:
        print
        print 'uploading offline-image ...',
        fp = open('offline.jpg', 'rb')
        ftp = ftplib.FTP(host, user, passwd)
        ftp.storbinary('STOR ' + filename, fp, 8192)
        ftp.close()
        fp.close()
        print 'done'
        print 'Program terminated - now it is save to close the window.'
        sys.exit()
